#include "Ambiente.h"

Ambiente::Ambiente(std::string nomeFundo) {
    sair = false;
    etapa = 0;
    contador = 0;
    std::string arquivoFundo = "imagens//" + nomeFundo + ".bmp";
    fundo = load_bitmap(arquivoFundo.c_str(), NULL);
    if (!fundo) {
        fundo = create_bitmap(
                    MAPA_LARGURA * TAMANHO_CELULA,
                    MAPA_ALTURA * TAMANHO_CELULA);
        clear_to_color(fundo, makecol(40, 180, 70));
    }
    folha = create_bitmap(
                MAPA_LARGURA * TAMANHO_CELULA,
                MAPA_ALTURA * TAMANHO_CELULA);
}

void Ambiente::desenhar() {
    show_mouse(NULL);
    draw_sprite(folha, fundo, 0, 0);
    for (int x = 0; x < MAPA_LARGURA; x++) {
        for (int y = 0; y < MAPA_ALTURA; y++) {
            if (mapa[x][y] != NULL) {
                mapa[x][y]->desenhar(etapa);
            }
        }
    }
    show_mouse(folha);
    blit(folha, screen, 0, 0, 0, 0, MAX_X, MAX_Y);
}

void Ambiente::executar() {
    while (!sair) {
        if (etapa == 0) {
            for (int x = 0; x < MAPA_LARGURA; x++) {
                for (int y = 0; y < MAPA_ALTURA; y++) {
                    if (mapa[x][y] != NULL) {
                        if (mapa[x][y]->paraApagar()) {
                            delete mapa[x][y];
                            mapa[x][y] = NULL;
                        } else {
                            mapa[x][y]->rodar();
                        }
                    }
                }
            }
        }

        desenhar();
        rest(100);
        etapa = (etapa = 0) ? etapa = 1 : etapa = 0;
        contador++;
        if (key[KEY_ESC]) {
            sair = true;
        }
    }
}

BITMAP* Ambiente::getFolha() {
    return folha;
}

Elemento* Ambiente::getElemento(int x, int y) {
    if ((x < 0) || (x >= MAPA_LARGURA) ||
            (y < 0) || (y >= MAPA_ALTURA)) {
        return false;
    }
    return mapa[x][y];
}

long Ambiente::getContador() {
    return contador;
}

bool Ambiente::moverElemento(Elemento* e, int x, int y) {
    if ((mapa[x][y] != NULL) || (e == NULL)) {
        return false;
    }
    if ((x < 0) || (x >= MAPA_LARGURA) ||
            (y < 0) || (y >= MAPA_ALTURA)) {
        return false;
    }
    int auxX = e->getX();
    int auxY = e->getY();
    mapa[x][y] = e;
    e->setX(x);
    e->setY(y);
    mapa[auxX][auxY] = NULL;
    return true;
}

bool Ambiente::inserirElemento(Elemento* e) {
    if (e == NULL) {
        return false;
    }
    if (mapa[e->getX()][e->getY()] != NULL) {
        return false;
    }
    if ((e->getX() < 0) || (e->getX() >= MAPA_LARGURA) ||
            (e->getY() < 0) || (e->getY() >= MAPA_ALTURA)) {
        return false;
    }
    mapa[e->getX()][e->getY()] = e;
    return true;
}

Ambiente::~Ambiente() {
    for (int x = 0; x < MAPA_LARGURA; x++) {
        for (int y = 0; y < MAPA_ALTURA; y++) {
            if (mapa[x][y] != NULL) {
                delete mapa[x][y];
            }
        }
    }
}
