#include "Ambiente.h"
#include "Criatura.h"

int timer;
void increase_timer(void) {
    timer++;
}
END_OF_FUNCTION(increase_timer);

class Main {
private:
    Ambiente *ambiente;
public:
    Main();
    void executar();
    void lerMapa(std::string nomeMapa);
    virtual ~Main();
};

Main::Main() {
    allegro_init();
    install_keyboard();
    install_timer();
    install_mouse();
    install_sound(SOUND_CARD, MIDI_CARD, NULL);
    set_color_depth(16);
    set_volume(128, 255);
    set_gfx_mode(VMODE, MAX_X, MAX_Y, 0, 0);
    timer = 0;
    LOCK_VARIABLE(timer);
    LOCK_FUNCTION(increase_timer);
    install_int(increase_timer, 1000);

    ambiente = new Ambiente("fundo");
    lerMapa("blocos");
}

void Main::executar() {
    ambiente->executar();
}

void Main::lerMapa(std::string nomeMapa) {
    FILE *file;
    std::string nomeArquivo = nomeMapa + ".map";
    if ((file = fopen(nomeArquivo.c_str(), "r")) == NULL) {
        std::cout << "Arquivo " << nomeArquivo << " nao pode ser aberto." << std::endl;
        exit(1);
    }
    int x = 0;
    int y = 0;
    while (!feof(file)) {
        Elemento* e = NULL;
        bool caracterValido = true;
        int item = fgetc(file);
        switch (item) {
        case '0':
            break;
        case '1':
            e = new Elemento("parede", ambiente->getFolha(), x, y);
            break;
        case '2':
            e = new Criatura("monstro", ambiente, ambiente->getFolha(), x, y);
            break;
        case '3':
            e = new Criatura("avatar", ambiente, ambiente->getFolha(), x, y);
            break;
        case '4':
            e = new Elemento("arvore", ambiente->getFolha(), x, y);
            break;
        default:
            caracterValido = false;
        }
        if (caracterValido) {
            if (e != NULL) {
                ambiente->inserirElemento(e);
            }
            x++;
            if (x >= 16) {
                x = 0;
                y++;
            }
        }
    }
    fclose(file);
}

Main::~Main() {
    delete ambiente;
    stop_midi();
    remove_int(increase_timer);
    remove_timer();
    fade_out(4);
    allegro_exit();
}

int main() {
    Main* main = new Main();
    main->executar();
    delete main;
    return 0;
}
END_OF_MAIN();
